-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
-- P8. Tutorial on the structure of a CLK generator:
-- OSC_CLK_in = 50 MHz  Intel - Altera DE10-Lite 
-- CLK_A_SQ, high frequency (5 MHz). You can measure using other values 
-- CLK_B_SQ, low frequency (10 kHz)

-- Plan: Frequency dividers and T_FF to square the pulsed terminal count signals    
-- Count Enable (CE) and asynchronous active-high clear direct (CD)
-- 2 cascaded frequency dividers in series 
-- 2 T_FF to square the pulsed waveforms 
-- The reference schematic translated here into VHDL is Fig. 3 in: 
-- https://digsys.upc.edu/csd/P08/L08_2/L8_2.html
--------------------------------------------------------------------------------
LIBRARY IEEE;
   USE IEEE.STD_LOGIC_1164.ALL;
   USE IEEE.STD_LOGIC_ARITH.ALL;
   USE IEEE.STD_LOGIC_UNSIGNED.ALL;
	
ENTITY CLK_Generator IS  
      PORT(	CD,CE			: 	IN  std_logic;
			OSC_CLK_in		: 	IN  std_logic;
			CLK_A_SQ		:	OUT std_logic;
			CLK_B_SQ		:	OUT std_logic
          );
END CLK_Generator ;

ARCHITECTURE schematic OF CLK_Generator  IS  
-- Components
	COMPONENT Freq_Div_500 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC500		:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT Freq_Div_5 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC5			:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT T_FF IS
		Port (	
			CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			T 		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC
		);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL CLK				:	std_logic;
	SIGNAL Pulsed_2A		:	std_logic;
	SIGNAL Pulsed_2B		:	std_logic;
		
	BEGIN  
-- Instantiation of components

	Chip1	: 	Freq_Div_5
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> CE,
			TC5		=> Pulsed_2A					  
		);

	Chip2	:  Freq_Div_500
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> Pulsed_2A,
			TC500 	=> Pulsed_2B					  
		);

	Chip3	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2A,
			Q 		=> CLK_A_SQ				  
		);		

	Chip4	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2B,
			Q 		=> CLK_B_SQ					  
		);		
			
-- Connections and logic between components

	-- The internal CLK form the external quartz crystal 
	CLK <= OSC_CLK_in;

END schematic ;
